# -*- coding: utf-8 -*-
import argparse

from pyscaffold.api import Extension, helpers
from pyscaffold.extensions.no_skeleton import NoSkeleton
from pyscaffold.extensions.pre_commit import PreCommit

from pyscaffoldext.markdown.extension import MarkDown

from . import templates


class IncludeExtensions(argparse.Action):
    """Activate other extensions
    """

    def __call__(self, parser, namespace, values, option_string=None):
        extensions = [
            NoSkeleton("no_skeleton"),
            PreCommit("pre_commit"),
            DSProjectVSCode("dsproject_vscode"),
        ]
        namespace.extensions.extend(extensions)


class DSProjectVSCode(Extension):
    """Template for data-science projects with support for VS Code container development
    """

    def augment_cli(self, parser):
        """Augments the command-line interface parser

        A command line argument ``--FLAG`` where FLAG=``self.name`` is added
        which appends ``self.activate`` to the list of extensions. As help
        text the docstring of the extension class is used.
        In most cases this method does not need to be overwritten.

        Args:
            parser: current parser object
        """
        help = self.__doc__[0].lower() + self.__doc__[1:]

        parser.add_argument(
            self.flag, help=help, nargs=0, dest="extensions", action=IncludeExtensions
        )
        return self

    def activate(self, actions):
        actions = self.register(actions, add_dsproject_vscode, after="define_structure")
        actions = self.register(actions, replace_readme, after="add_dsproject_vscode")
        return actions


def add_dsproject_vscode(struct, opts):
    """Adds basic module for custom extension

    Args:
        struct (dict): project representation as (possibly) nested
            :obj:`dict`.
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        struct, opts: updated project representation and options
    """
    gitignore_all = templates.gitignore_all(opts)
    gitkeep = templates.gitkeep(opts)

    path = [opts["project"], "data", ".gitignore"]
    struct = helpers.ensure(
        struct, path, templates.gitignore_data(opts), helpers.NO_OVERWRITE
    )
    for folder in ("external", "interim", "preprocessed", "raw"):
        path = [opts["project"], "data", folder, ".gitignore"]
        struct = helpers.ensure(struct, path, gitignore_all, helpers.NO_OVERWRITE)

    path = [opts["project"], "data", "output", ".gitkeep"]
    struct = helpers.ensure(struct, path, gitkeep, helpers.NO_OVERWRITE)

    path = [opts["project"], "notebooks", "template.ipynb"]
    template_ipynb = templates.template_ipynb(opts)
    struct = helpers.ensure(struct, path, template_ipynb, helpers.NO_OVERWRITE)

    path = [opts["project"], "scripts", "train_model.py"]
    train_model_py = templates.train_model_py(opts)
    struct = helpers.ensure(struct, path, train_model_py, helpers.NO_OVERWRITE)

    path = [opts["project"], "models", ".gitignore"]
    struct = helpers.ensure(struct, path, gitignore_all, helpers.NO_OVERWRITE)

    path = [opts["project"], "references", ".gitignore"]
    struct = helpers.ensure(struct, path, "", helpers.NO_OVERWRITE)

    path = [opts["project"], "reports", "figures", ".gitignore"]
    struct = helpers.ensure(struct, path, "", helpers.NO_OVERWRITE)

    # path = [opts["project"], ".devcontainer", "devcontainer.json"]
    # devcontainer_json = templates.devcontainer_json(opts)
    # struct = helpers.ensure(struct, path, devcontainer_json, helpers.NO_OVERWRITE)

    path = [opts["project"], ".devcontainer", "devcontainer.local.json"]
    devcontainer_local_json = templates.devcontainer_local_json(opts)
    struct = helpers.ensure(struct, path, devcontainer_local_json, helpers.NO_OVERWRITE)

    path = [opts["project"], ".devcontainer", "devcontainer.remote.json"]
    devcontainer_remote_json = templates.devcontainer_remote_json(opts)
    struct = helpers.ensure(struct, path, devcontainer_remote_json, helpers.NO_OVERWRITE)

    path = [opts["project"], ".devcontainer", "Dockerfile.dev.base"]
    dockerfile_dev_base = templates.dockerfile_dev_base(opts)
    struct = helpers.ensure(struct, path, dockerfile_dev_base, helpers.NO_OVERWRITE)

    path = [opts["project"], ".vscode", "settings.json"]
    settings_json = templates.settings_json(opts)
    struct = helpers.ensure(struct, path, settings_json, helpers.NO_OVERWRITE)

    path = [opts["project"], "environment.dev.base.yml"]
    environment_dev_base_yml = templates.environment_dev_base_yml(opts)
    struct = helpers.ensure(struct, path, environment_dev_base_yml, helpers.NO_OVERWRITE)

    path = [opts["project"], "docker-compose.yml"]
    docker_compose_yml = templates.docker_compose_yml(opts)
    struct = helpers.ensure(struct, path, docker_compose_yml, helpers.NO_OVERWRITE)

    path = [opts["project"], "docker-compose.remote.yml"]
    docker_compose_remote_yml = templates.docker_compose_remote_yml(opts)
    struct = helpers.ensure(struct, path, docker_compose_remote_yml, helpers.NO_OVERWRITE)

    path = [opts["project"], "path.env"]
    path_env = templates.path_env(opts)
    struct = helpers.ensure(struct, path, path_env, helpers.NO_OVERWRITE)

    path = [opts["project"], "requirements.txt"]
    struct = helpers.reject(struct, path)

    path = [opts["project"], "configs", ".gitignore"]
    struct = helpers.ensure(struct, path, "", helpers.NO_OVERWRITE)
    return struct, opts


def replace_readme(struct, opts):
    """Replace the readme.md of the markdown extension by our own

    Args:
        struct (dict): project representation as (possibly) nested
            :obj:`dict`.
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        struct, opts: updated project representation and options
    """
    # let the markdown extension do its job first
    struct, opts = MarkDown("markdown").markdown(struct, opts)

    file_path = [opts["project"], "README.md"]
    struct = helpers.reject(struct, file_path)
    readme = templates.readme_md(opts)
    struct = helpers.ensure(struct, file_path, readme, helpers.NO_OVERWRITE)
    return struct, opts



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Dummy conftest.py for custom_extension.

    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    https://pytest.org/latest/plugins.html
"""
from __future__ import absolute_import, division, print_function

import os
import shlex
import stat
from shutil import rmtree

import pytest


def set_writable(func, path, exc_info):
    if not os.access(path, os.W_OK):
        os.chmod(path, stat.S_IWUSR)
        func(path)
    else:
        raise RuntimeError


@pytest.fixture
def tmpfolder(tmpdir):
    old_path = os.getcwd()
    newpath = str(tmpdir)
    os.chdir(newpath)
    try:
        yield tmpdir
    finally:
        os.chdir(old_path)
        rmtree(newpath, onerror=set_writable)


@pytest.fixture
def venv(virtualenv):
    """Create a virtualenv for each test"""
    return virtualenv


@pytest.fixture
def venv_run(venv):
    """Run a command inside the venv"""

    def _run(*args, **kwargs):
        # pytest-virtualenv doesn't play nicely with external os.chdir
        # so let's be explicit about it...
        kwargs["cd"] = os.getcwd()
        kwargs["capture"] = True
        if len(args) == 1 and isinstance(args[0], str):
            args = shlex.split(args[0])
        return venv.run(args, **kwargs).strip()

    return _run



# -*- coding: utf-8 -*-
import os
import string
from pkg_resources import resource_string

from .. import __version__ as dsproject_version


def get_template(name):
    """Retrieve the template by name

    Args:
        name: name of template

    Returns:
        :obj:`string.Template`: template
    """
    file_name = "{name}.template".format(name=name)
    data = resource_string(__name__, file_name)
    # we assure that line endings are converted to '\n' for all OS
    data = data.decode(encoding="utf-8").replace(os.linesep, "\n")
    return string.Template(data)


def gitignore_all(opts):
    """gitignore file that ignores just everything

    Ignore everything except of this gitignore file.

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("gitignore_all")
    return template.safe_substitute(opts)


def gitignore_data(opts):
    """gitignore file that ignores almost everything

    Ignore everything except of gitignore also in sub directories.

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("gitignore_data")
    return template.safe_substitute(opts)


def gitkeep(opts):
    """.gitkeep file that keeps directory

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("gitkeep")
    return template.safe_substitute(opts)


def readme_md(opts):
    """Adds a basic README.md

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("readme_md")
    opts["pkg"] = opts["package"].ljust(19)
    opts["dsproject_version"] = dsproject_version
    return template.safe_substitute(opts)


def template_ipynb(opts):
    """Adds a template Jupyter notebook

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("template_ipynb")
    return template.safe_substitute(opts)


def train_model_py(opts):
    """Adds a template python experiment

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("train_model_py")
    return template.safe_substitute(opts)


def devcontainer_json(opts):
    """devcontainer.json for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("devcontainer_json")
    return template.safe_substitute(opts)


def devcontainer_local_json(opts):
    """devcontainer.local.json for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("devcontainer_local_json")
    return template.safe_substitute(opts)


def devcontainer_remote_json(opts):
    """devcontainer.remote.json for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("devcontainer_remote_json")
    return template.safe_substitute(opts)


def docker_compose_yml(opts):
    """docker-compose.yml for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("docker_compose_yml")
    return template.safe_substitute(opts)


def docker_compose_remote_yml(opts):
    """docker-compose.remote.yml for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("docker_compose_remote_yml")
    return template.safe_substitute(opts)


def dockerfile_dev_base(opts):
    """Dockerfile.dev.base for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("dockerfile_dev_base")
    return template.safe_substitute(opts)


def environment_dev_yml(opts):
    """environment.dev.yml for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("environment_dev_yml")
    return template.safe_substitute(opts)


def environment_dev_base_yml(opts):
    """environment.dev.base.yml for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("environment_dev_base_yml")
    return template.safe_substitute(opts)


def path_env(opts):
    """path.env for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("path_env")
    return template.safe_substitute(opts)


def settings_json(opts):
    """settings.json for vscode

    Args:
        opts (dict): given options, see :obj:`create_project` for
            an extensive list.

    Returns:
        str: file content as string
    """
    template = get_template("settings_json")
    return template.safe_substitute(opts)



# -*- coding: utf-8 -*-
from os.path import exists as path_exists

from pyscaffold.api import create_project
from pyscaffold.cli import parse_args, process_opts
from pyscaffold.utils import chdir

EXT_FLAG = "--dsproject-vscode"
FLAKE8 = "flake8 --max-line-length=88"


def test_add_custom_extension(tmpfolder):
    args = [EXT_FLAG, "my_project", "-p", "my_package"]
    opts = parse_args(args)
    opts = process_opts(opts)
    create_project(opts)

    assert path_exists("my_project/src/my_package/__init__.py")


def test_generated_extension_flake8(tmpfolder, venv_run):
    args = [EXT_FLAG, "my_project"]
    opts = parse_args(args)
    opts = process_opts(opts)
    create_project(opts)

    with chdir("my_project"):
        assert "" == venv_run(FLAKE8)
        venv_run("python setup.py install")

    venv_run("putup {ext_flag} the_actual_project".format(ext_flag=EXT_FLAG))
    assert path_exists("the_actual_project/setup.cfg")

    with chdir("the_actual_project"):
        assert "" == venv_run(FLAKE8)
